varying highp vec2 v_texCoord;
varying highp vec2 v_texCoordRounded;

uniform highp sampler2D u_texture;  // Template (for R)
uniform highp sampler2D u_texture1; // Template (for GB)
uniform highp sampler2D u_texture2; // Palette

uniform highp vec4 u_colorableImageSize;

const int convSize = 11;
const mediump float convSizeFloat = float(convSize);
const int numSamples = convSize * convSize;
const mediump float leftLimit = float(-(convSize-1)/2);

void main() {
	highp vec2 tStep = 1.0 / u_colorableImageSize.xy;
	
	mediump float count = 0.0;
	mediump float gray = 0.0;
	highp vec3 colorInfo = vec3(0.0, 0.0, 0.0);
	for (int i = 0; i < numSamples; i++) {
		mediump float iFloat = float(i);
		mediump float sampleX = leftLimit + mod(iFloat, convSizeFloat);
		mediump float sampleY = leftLimit + (iFloat / convSizeFloat);
		highp vec2 sampleValue = vec2(sampleX * tStep.x, sampleY * tStep.y);
		
		gray += texture2D(u_texture, v_texCoord + sampleValue).r;
		highp vec2 paletteCoord = texture2D(u_texture1, v_texCoord + sampleValue).bg;
		if (paletteCoord != vec2(0.0, 0.0)) {
			colorInfo += texture2D(u_texture2, paletteCoord).rgb;
			count++;
		}
	}

	gl_FragColor = vec4((colorInfo / count) * (gray / float(numSamples)), 1.0);
}